function [bestBox, E] = ml_findBox1f(kpAssign, imH, imW, pyramidBinWeights, ...
    nBranch, nLevel, szRange, tolFac)
% function [bestBox, E] = ml_findBox1f(kpAssign, imH, imW, pyramidBinWeights, ...
%    nBranch, nLevel, szRange, tolFac)
% Find the best box in an image minimize a pyramid of weights.
%    like in the pyrmaid matching kernel by minimzing the objective:
%    sum_ij weights(i,j)*hist_i(j)
% Inputs:
%   kpAssign: a matrix for histogram bin assignments of keypoints. 
%       kpAssign(i,1), kpAssign(i,2), kpAssign(i,3) are the row#, col#,
%       bin#. 1 <= row# <= imH; 1 <= col# <= imW; 1 <= bin# <= k, where k
%       is the length of binWeights.
%   imH, imW: height and width of the image
%   pyramidBinWeights: a k*m array of weights for histogram bins, 
%       weights can be negative. 
%       Here m = [pow(nBranch, nLevel + 1)- 1]/(nBranch - 1)
%       The first vector is the for root level.
%       The tree has nBranch, 
%       the next [pow(nBranch, nLevel)- 1]/(nBranch - 1) weight vectors 
%       are for the first branch and so on so forth. 
%   nBranch, nLevel: number of branches and levels. nLevel = 0 means no
%       pyrmaid. This would be similar to the ml_findBox1b
%	szRange: a 4*1 vector for [wmin; wmax; hmin; hmax], range for width and height
%   tolFac: for stopping early. tolFac is from 0 to 1. 
%       tolFactor = 0: stop when global optimum is reach
%       tolFac > 0: stop when the difference between the global optimum
%           and the best known is < tolFac of the best known value.
% Output:
%   bestBox: a 4*1 vector for the left, top, right, bottom of the rectangle
%   E: energy
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 21 Dec 2008

% Check szRange
if exist('szRange','var') && ~isempty(szRange)
    if (size(szRange, 1) ~= 4) || (size(szRange, 2) ~= 1)
        error('ml_findBox1f.m: szRange should be a 4*1 vector');
    end;

    if (szRange(1) > szRange(2)) || (szRange(3) > szRange(4)) ||...
        szRange(2) <= 0 || szRange(4) <=0 || ...
        szRange(1) > imW || szRange(3) > imH
        error('ml_findBox1f.m: invalid size range');
    end;
else
    szRange = [0; imW; 0; imH];
end;

% Check  tolFac
if ~exist('tolFac', 'var') || isempty(tolFac)
    tolFac = 0;
end;

if (tolFac > 1) || (tolFac < 0)
    error('ml_findBox1f.m: tolFac must be from 0 to 1');
end;

% Check consistency between # weight vectors and nBranch, nLevel
[nHistBin, m] = size(pyramidBinWeights);
if m ~= (nBranch^(nLevel + 1) - 1)/(nBranch - 1)
    error('ml_findBox1f.m: mismatch b/t # of weight vectors and nBranch, nLevel');
end;

% Check kpAssign
if sum(kpAssign(:) <= 0) > 0
    error('entries of kpAssign should be positive integers');
end
if sum(kpAssign(:,1) > imH) > 0
    error('entries of kpAssign(:,1) must be <= imH');
end;
if sum(kpAssign(:,2) > imW) > 0
    error('entries of kpAssign(:,2) must be <= imW');
end;
if sum(kpAssign(:,3) > nHistBin) > 0
    error('entries of kpAssign(:,3) must be smaller than the length of binWeights');
end

% Run the actual branh and bound code.
[bestBox, E] = m_mexFindBox1f(kpAssign, imH, imW, pyramidBinWeights, ...
    nBranch, nLevel, szRange, tolFac); 
