function Segs = ml_findMulti1DSegs_seq(kpAssign, imW, binW, nSeg, szRng, thres)
% Segs = ml_findMulti1DSegs_seq(kpAssign, imW, binW, nSeg, szRng, thres)
% Finding multiple 1D segments from the 1D histograms.
% The feature vector representing a group of segments is a histogram, hist,
% which is the addition of all histograms. The objective is to minimize 
% sum_i {binW(i)*hist(i)}
% This function finds the segments sequentially and does not guarantee to find
% the global optimum.
% Inputs:
%   kpAssign: a n*2 matrix for histogram bin assignments of keypoints. 
%       kpAssign(i,1), kpAssign(i,2) are the col#, bin#. 
%       1 <= col# <= imW; 1 <= bin# <= k, where k
%       is the length of binWeights.
%   imW: width or length of 1D signal.
%   binW: a k*1 vector of weights for histogram bins, weights can be
%       negative
%   nSeg: number of segments to extract. The default is 1. If nSeg is -1,
%       segments are sequentially extracted until any newly addition will
%       carry a weighted histogram > thres. thres is another input param of
%       this function.
%	szRng: a 2*1 vector for [minLength; maxLength], range for length of
%       the individual segment
%   thres: see explanation for nSeg.
% Output:
%   Segs: a 3*K matrix with K <= nSeg. Segs([1,2],i) marks the left and
%   right ends of the interval. Segs(3,i) is the value of the interval.
% Examples:
%   signal_1D = ones(1,40);
%   signal_1D([4:7, 10:15, 25:35]) = 2;
%   n = length(signal_1D);
%   kpAssign = [(1:n)', signal_1D(:)];
%   ml_findMulti1DSegs_seq(kpAssign, n, [1; -1], 3)
%   ml_findMulti1DSegs_seq(kpAssign, n, [1; -1], 2)
%   ml_findMulti1DSegs_seq(kpAssign, n, [1; -2], 2, [5; 10])
%   ml_findMulti1DSegs_seq(kpAssign, n, [1; -1], -1, [5; 10], -5)
%   ml_findMulti1DSegs_seq(kpAssign, n, [1; -1], -1, [5; 10], -7)
% See also:
%   ml_findMulti1DSegs.m: which guarantees to find global minimum, but
%       cannot incorporate size constraints. 
%   
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 13 Jan 2009
% Last modified: 31 Jan 2009

if size(binW, 2) ~= 1
    error('binWeights should be a column vector');
end;

nHistBin = size(binW, 1);

% Check kpAssign
if size(kpAssign, 2) ~= 2
    error('kpAssign must be n*2 matrix');
end;

if sum(kpAssign(:) <= 0) > 0
    error('entries of kpAssign should be positive integers');
end

if sum(kpAssign(:,1) > imW) > 0
    error('entries of kpAssign(:,1) must be <= imW');
end;
if sum(kpAssign(:,2) > nHistBin) > 0
    error('entries of kpAssign(:,2) must be smaller than the length of binWeights');
end


if exist('szRng','var') && ~isempty(szRng)
    if (size(szRng, 1) ~= 2) || (size(szRng, 2) ~= 1)
        error('szRng should be a 2*1 vector');
    end;

    if (szRng(1) > szRng(2)) ||  szRng(2) <= 0 || szRng(1) > imW
        error('invalid size range');
    end;
else
    szRng = [0; imW];
end;

if ~exist('thres', 'var') || isempty(thres)
    thres = 0;
end;

Segs = m_mexFindMulti1DSegs_seq(kpAssign, imW, binW, nSeg, szRng, thres);



