function [Segs, nIter] = ml_findNormMulti1DSegs(pixVals, nSeg, alpha, beta, tol)
% [Segs, nIter] = ml_findNormMulti1DSegs(pixVals, nSeg, alpha, beta, tol)
% Given a temporal signal a_1,...,a_n, find multiple 1D segments S such that: 
%   (sum_(i in S) a_i + alpha)/(length(S) + beta)
% This function find GLOBAL minimum up to some controllable tollerance. 
% Inputs:
%   pixVals: a n*1 vector for temporal signal a.
%   nSeg: number of segments to extract, must a pos integer. The default is 1
%   alpha: constant at the enumerator, alpha could be either positve or negative
%       bigger alpha leads to longer sequence. 
%   beta: constant at the denominator, beta must be a strictly positive number.
%       the bigger beta, the less important the length, which entails longer
%       segment.
%   tol: the returned k-segmentation is guaranteed to be within tol of the
%       global optimum value.
% Output:
%   Segs: a 3*K matrix with K <= nSeg. Segs([1,2],i) marks the left and
%       right ends of the interval. Segs(3,i) is the value of the interval.
%   nIter: # of iterations
% Examples:
%   pixVals = ones(40,1);
%   pixVals([4:7, 10:15, 25:35]) = -1;
%   [Segs, nIter] = ml_findNormMulti1DSegs(pixVals, 2, 0, 1000, 1e-10)   % [25 35; 4 15]
%   [Segs, nIter] = ml_findNormMulti1DSegs(pixVals, 2, 0, 10, 1e-10)     % [25 35, 10 15]
%   [Segs, nIter] = ml_findNormMulti1DSegs(pixVals, 3, 0, 1000000, 1e-2) % [25 35]
%   [Segs, nIter] = ml_findNormMulti1DSegs(pixVals, 3, 0, 1000000, 1e-10)% [25 35; 10 15; 4 7]; 
% See also:
%   ml_findMulti1DSegs.m, ml_findMulti1DSegs2.m 
%   ml_findMulti1DSegs_seq.m, ml_findMulti1DSegs_seq2.m
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 14 Aug 2009

if size(pixVals, 2) ~= 1
    error('pixVals must be a column vector');
end

if (nSeg < 1)
    error('Number of segment requested must be at least 1');
end;

if (beta <= 0)
    error('beta must be a strictly positive number');
end;

if (tol <= 0)
    error('tol must be a positive number');
end;

[Segs, nIter] = m_mexFindNormMulti1DSegs(pixVals, nSeg, alpha, beta, tol);



