function [label, energy] = ml_multiLabelKSeg(pixVals, nSeg, segSwtchInd)
% function [label, energy] = ml_multiLabelKSeg(pixVals, nSeg)
% function [label, energy] = ml_multiLabelKSeg(pixVals, nSeg, segSwtchInd)
% Divide a segment into at most nSeg sub-segments
% and assign labels to each segments such that the sum of assigned elements are minimized.
% This function find GLOBAL minimum using dynamic programming
% Inputs:
%   pixVals: a imH*nLabel matrix
%   nSeg: number of maximum sub-segments, must be a positive integer. 
%   segSwtchInd: nLabel*nLabel matrix indicating the jump from one segment to the next.
%       segSwtchInd(i,j) = 1, jumping from label i to j corresponds to 2 different
%       segSwtchInd(i,j) = 0, jumping from label i to j does not correspond to two
%           different segments
%       E.g., for AU deteciton: 0-neutral, 1-onset, 2-offset; the indicator for segment
%       switching is: A(i,i) = 0; A(0,1) = 1; A(0,2) = 1; A(1,0) = 1; A(1,2) = 0; A(2,0) =
%       1; A(2,1) = 1; Going from onset to offset states means the same segment.
%       Default value: segSwtchInd(i,j) = 0 iff i=j. 
%       
% Output:
%   label: a imH*1 matrix, label(i) is the label for frame i^th.
% Examples:
%   pixVals = ones(40,1);
%   pixVals([4:7, 10:15, 25:35]) = -1;
%   pixVals(:,2) = 0;
%   pixVals(:,3) = 3;
%   pixVals(10:15,3) = -2;
%   [label1, energy1] = ml_multiLabelKSeg(pixVals, 4);
%   [label2, energy2] = ml_multiLabelKSeg(pixVals, 4, []);
%   A3 = [0 1 1; 1 0 1; 1 1 0];
%   [label3, energy3] = ml_multiLabelKSeg(pixVals, 4, A3);
%   A4 = [0 1 1; 1 0 0; 1 1 0]; % similar to AU detection problem
%   [label4, energy4] = ml_multiLabelKSeg(pixVals, 4, A4);
%   % Expecitng: energy1,energy2,energy3 should be the same, energy4 should be lower.
% By: Minh Hoai Nguyen (minhhoai@gmail.com)
% Date: 1 Dec 2009
% Last modified: 12 May 2010


if (nSeg < 1)
    error('Number of segment requested must be at least 1');
end;

if ~exist('segSwtchInd', 'var') || isempty(segSwtchInd)
    [Segs, energy] = m_mexMultiLabelKSeg(pixVals, nSeg);
    label = zeros(size(pixVals,1),1);
    for i=1:size(Segs,2)
        label(Segs(1,i):Segs(2,i)) = Segs(3,i);
    end;
else
    [label, energy] = m_mexMultiLabelKSeg2(pixVals, nSeg, segSwtchInd);
end;
    